﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Web;
using ThinkGeo.MapSuite.Core;

namespace CustomScaleBar
{
    public class CustomScaleBar : ScaleBarAdornmentLayer
    {
        private int width = 200;
        private string unitText;
        private double meterToUnit;
        private double unitRoundValue;
        private ManagedProj4Projection proj4;

        //Maximum width of the scale bar in screen coordinate
        public int MaxWidth
        {
            get { return width; }
            set { width = value; }
        }

        //Text to be displayed for the unit.
        public string UnitText
        {
            get { return unitText; }
            set { unitText = value; }
        }

        //Ratio of meter to the unit.
        public double MeterToUnit
        {
            get { return meterToUnit; }
            set { meterToUnit = value; }
        }

        //Projection for the map (internal) and the projection the scale bar is based on (external).
        public ManagedProj4Projection Projection
        {
            get { return proj4; }
            set { proj4 = value; }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            //Set the position of the scale bar on canvas.
            float Xpos = 15;
            float Ypos = canvas.Height - 40;

            //Gets the left and right location of the scale bar in world coordinate according to the maximum width and X and Y position.
            ScreenPointF screenLocation = GetDrawingLocation(canvas, width, Ypos);
            PointShape scaleBarMapPointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, screenLocation.X, screenLocation.Y, canvas.Width, canvas.Height);
            PointShape scaleBarMapRightPointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, screenLocation.X + this.width, screenLocation.Y, canvas.Width, canvas.Height);

            //Applies the reprojection so that the scale bar is not based on the displayed projection of the map (internal) but on the desired projection (external)
            //Typically the external projection will have distance characteristics more accurate than the displayed projection for a more reliable scalebar.
            proj4.Open();
            PointShape scaleBarMapPointShapeGeo = (PointShape)proj4.ConvertToExternalProjection(scaleBarMapPointShape);
            PointShape scaleBarMapRightPointShapeGeo = (PointShape)proj4.ConvertToExternalProjection(scaleBarMapRightPointShape);
            proj4.Close();

            //Gets the length of the scale bar according to the unit and the maximum width of the scale bar.
            double fullBarLength = scaleBarMapPointShapeGeo.GetDistanceTo(scaleBarMapRightPointShapeGeo,GeographyUnit.DecimalDegree, DistanceUnit.Meter);
            //Adjusts the length of the scale bar in order to have a round number.
            unitRoundValue = GetRoundValue(fullBarLength / meterToUnit);
            double barLength = ((unitRoundValue * meterToUnit) * this.width) / fullBarLength;

            //Draw the line of the scale bar according to the adjusted length.
            GeoPen pen = new GeoPen(GeoColor.StandardColors.Black, 2F);
            canvas.DrawLine(new ScreenPointF[]  {new ScreenPointF(Xpos, Ypos - 10), new ScreenPointF(Xpos, Ypos), new ScreenPointF((float)barLength + Xpos, 
                        Ypos), new ScreenPointF((float)barLength + Xpos, Ypos - 10) }, pen, DrawingLevel.LevelOne, 0, 0);

            //Displays the text for the value and unit text.
            canvas.DrawText(Convert.ToString(unitRoundValue) + " " + unitText, new GeoFont("Arial", 8,DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.StandardColors.Black),
                new ScreenPointF[] { new ScreenPointF((float)(barLength / 2) + Xpos, Ypos - 10) }, DrawingLevel.LevelOne);

        }

        //Function to round down the length to fit within the maximum width of the scale bar.
        private double GetRoundValue(double unitValue)
        {
            double result;
            double interval = GetRoundingValue(unitValue);
            result = FloorNumber(unitValue, interval);
            return result;
        }

        //Gets the rounding value to be used according to the length of the unit value.
        private double GetRoundingValue(double UnitValue)
        {
            double result = 0;
            if (UnitValue > 100000) { result = 50000; }
            else if (UnitValue > 10000) { result = 5000; }
            else if (UnitValue > 1000) { result = 500; }
            else if (UnitValue > 100) { result = 50; }
            else if (UnitValue > 10) { result = 5; }
            else if (UnitValue > 1) { result = 0.5; }
            else if (UnitValue > 0.1) { result = 0.05; }
            else if (UnitValue > 0.01) { result = 0.005; }

            return result;
        }

        //Function used for rounding down a number according to the rounding value.
        private double FloorNumber(double Number, double Rounding)
        {
            double result = 0;
            double IEEERemainder = Math.IEEERemainder(Number, Rounding);
            if (IEEERemainder > 0)
                result = Number - IEEERemainder;
            else if (IEEERemainder < 0)
                result = (Number + Math.Abs(IEEERemainder)) - Rounding;
            else
                result = Number;
            return result;
        }
    }
}
